/*
Copyright 2007 Daniel Gonzalez Peña, Florentino Fernandez Riverola


This file is part of the AIBench Project. 

AIBench Project is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

AIBench Project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser Public License for more details.

You should have received a copy of the GNU Lesser Public License
along with AIBench Project.  If not, see <http://www.gnu.org/licenses/>.
 */

/*  
 * CoreUtils.java
 *
 * Created inside the SING research group (http://sing.ei.uvigo.es)
 * University of Vigo
 *
 * Created on 13/10/2008
 */
package es.uvigo.ei.aibench.core;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import es.uvigo.ei.aibench.core.clipboard.ClipboardItem;

/**
 * @author Daniel Glez-Peña
 * 
 */
public class CoreUtils {
	// TODO �abstraer todos los createParams() en uno solo??

	public static ParamSpec[] createParams(List<?> params) {
		ParamSpec[] paramsSpec = new ParamSpec[params.size()];
		for (int p = 0; p < params.size(); p++)
			paramsSpec[p] = createParam(params.get(p));
		return paramsSpec;
	}

	private static ParamSpec[] createParams(Object[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(array[i]);
		return specArray;
	}

	private static ParamSpec[] createParams(int[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Integer(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(boolean[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Boolean(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(char[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Character(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(double[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Double(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(float[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Float(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(long[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Long(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(short[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Short(array[i]));
		return specArray;
	}

	private static ParamSpec[] createParams(byte[] array) {
		ParamSpec[] specArray = new ParamSpec[array.length];
		for (int i = 0; i < specArray.length; i++)
			specArray[i] = createParam(new Byte(array[i]));
		return specArray;
	}

	private static ParamSpec createParam(Object param) {
		if (param==null) return new ParamSpec("null", Object.class, null, ParamSource.CLIPBOARD);
		Class<?> clazz = param.getClass();
		
		if (clazz.isEnum()){
			return new ParamSpec("enum", clazz, param, ParamSource.ENUM);
			
		}
		else if (clazz.isArray()) {
			// Primero hay que descartar los tipos b�sicos para que no pete el
			// cast a Object[]
			if (clazz.equals((new int[0]).getClass()))
				return new ParamSpec("array", Integer[].class,
						createParams((int[]) param));
			if (clazz.equals((new byte[0]).getClass()))
				return new ParamSpec("array", Byte[].class,
						createParams((byte[]) param));
			if (clazz.equals((new short[0]).getClass()))
				return new ParamSpec("array", Short[].class,
						createParams((short[]) param));
			if (clazz.equals((new long[0]).getClass()))
				return new ParamSpec("array", Long[].class,
						createParams((long[]) param));
			if (clazz.equals((new float[0]).getClass()))
				return new ParamSpec("array", Float[].class,
						createParams((float[]) param));
			if (clazz.equals((new double[0]).getClass()))
				return new ParamSpec("array", Double[].class,
						createParams((double[]) param));
			if (clazz.equals((new char[0]).getClass()))
				return new ParamSpec("array", Character[].class,
						createParams((char[]) param));
			if (clazz.equals((new boolean[0]).getClass()))
				return new ParamSpec("array", Boolean[].class,
						createParams((boolean[]) param));
			return new ParamSpec("array", clazz, createParams((Object[]) param));
		} else {
			// Si no es array lo buscamos en el clipboard
			ClipboardItem item = Core.getInstance().getClipboard().getClipboardItem(param);
			if (item != null) {
				return new ParamSpec("item", clazz, item, ParamSource.CLIPBOARD);
			} else {
				// Si no, compruebo los tipos b�sicos...
				if (clazz.equals(Integer.class))
					return new ParamSpec("integer", Integer.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(String.class))
					return new ParamSpec("string", String.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Byte.class))
					return new ParamSpec("byte", Byte.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Short.class))
					return new ParamSpec("short", Short.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Long.class))
					return new ParamSpec("long", Long.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Float.class))
					return new ParamSpec("float", Float.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Double.class))
					return new ParamSpec("double", Double.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Character.class))
					return new ParamSpec("char", Character.class, param,
							ParamSource.STRING_CONSTRUCTOR);
				if (clazz.equals(Boolean.class))
					return new ParamSpec("boolean", Boolean.class, param,
							ParamSource.STRING_CONSTRUCTOR);

				if (Serializable.class.isAssignableFrom(clazz)) {
					return new ParamSpec("param", clazz, Base64Coder
							.encodeSerializableObject((Serializable) param),
							ParamSource.SERIALIZED);
				}
			}
		}
		return null;
	}
	
	public static boolean isValidURL(String url) {
		try {
			new URL(url);
			return true;
		} catch (MalformedURLException e) {
			return false;
		}
	}
	
	public static void openURL(String url) throws Exception {
		try {
			String osName = System.getProperty("os.name");
			if (osName.startsWith("Mac OS")) {
				Class<?> fileMgr = Class.forName("com.apple.eio.FileManager");
				Method openURL = fileMgr.getDeclaredMethod("openURL", new Class[] { String.class });
				openURL.invoke(null, new Object[] { url });
			} else if (osName.startsWith("Windows")) {
				Runtime.getRuntime().exec("rundll32 url.dll, FileProtocolHandler " + url);
			} else { // assume Unix or Linux
				String[] browsers = { "firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape" };
				String browser = null;
				for (int count = 0; count < browsers.length && browser == null; count++)
					if (Runtime.getRuntime().exec(new String[] { "which", browsers[count] }).waitFor() == 0)
						browser = browsers[count];
				if (browser == null)
					throw new Exception("Could not find web browser");
				else
					Runtime.getRuntime().exec(new String[] { browser, url });
			}
		} catch (Exception e) {
			Core.logger.warn("The url '" + url +"' could not be opened", e);
			throw e;
		}
	}
}
